@@ -33,12 +33,40 @@ class JournalBase:
         self.mentions: list[str] = []
 
     def __eq__(self, other) -> bool:
-        if isinstance(other, Journal):
+        if isinstance(other, JournalBase):
             return other.id == self.id
         elif isinstance(other, int):
             return other == self.id
         return False
 
+    def __gt__(self, other) -> bool:
+        if isinstance(other, JournalBase):
+            return self.id > other.id
+        elif isinstance(other, int):
+            return self.id > other
+        return False
+
+    def __ge__(self, other) -> bool:
+        if isinstance(other, JournalBase):
+            return self.id >= other.id
+        elif isinstance(other, int):
+            return self.id >= other
+        return False
+
+    def __lt__(self, other) -> bool:
+        if isinstance(other, JournalBase):
+            return self.id < other.id
+        elif isinstance(other, int):
+            return self.id < other
+        return False
+
+    def __le__(self, other) -> bool:
+        if isinstance(other, JournalBase):
+            return self.id <= other.id
+        elif isinstance(other, int):
+            return self.id <= other
+        return False
+
     def __iter__(self):
         yield "id", self.id
         yield "title", self.title
